from __future__ import division
from flask import Flask, request, session, render_template,Response
import Domain
from couchdbkit import Server, Document
import stripe
import json
from StringIO import StringIO
import re
import os
PORT = 8100 ;
path = os.environ['CONFIG']
f = open(path)
CONFIG = json.loads(f.read())
stripe_keys = {
    'secret_key': CONFIG['stripe']['secret'].strip(),
    'publishable_key': CONFIG['stripe']['pub'].strip()
}

stripe.api_key = stripe_keys['secret_key'].strip()


	
app = Flask(__name__)
"""
	This function will attempt to create an account for a user if the user does NOT exist
	Then Setup the stage for initialization of {signup for, user-account}
"""
COUCHDB = Server(uri=CONFIG['couchdb']['uri']) ;


@app.route('/init',methods=['POST'])
def init():
	DB = COUCHDB.get_db(CONFIG['couchdb']['db']) ;
	out_resp = '0'
	
	if 'User-Info' in request.headers:
		stream = request.headers.get('User-Info') ;
		user = json.loads(stream) ;
		session['user-info'] = user ;
		#
		# We should create the user in couchdb
		uid = user['uid'] ;
		session['uid'] = uid
		handler = Domain.User(DB,stripe) ;
		"""
			registering the user with us and making sure we can keep track of her basic operations & access to features
			At this point this is just a user i.e a consumer (not store owner), we will tally track on the type of user later on
		"""
		if handler.exists(uid) == False:
			
			#
			# We need to subscribe the user to the freemium services if any at all
			# Subscribing to a freemium plan hopefully this enables basic features of the application
			#
			free = []
			paid = []
			plans = [plan for plan in stripe.Plan.list().data if plan.amount == 0 	]					
			handler.init(uid,plans) ;
			
			out_resp = session['user-info']

		elif 'subscriptions' not in session:
			#
			# We have the user we should return the user's information
			#
			handler.init(uid)
			out_resp = handler.subscriptions()
		#
		# updating user-info with more information ...
		#
		session['user-info'] = handler.user;
		out_resp = CONFIG['stripe']['pub'].strip()
		# return '1'
	elif 'user-info' in session:
		out_resp = CONFIG['stripe']['pub'].strip()
	
	return out_resp
"""
	This function defines if a given user is a customer or not
	We should be able to tell by how we create customers
"""
@app.route('/checkout',methods=['GET'])
def is_customer ():
	bill = {"amount":0,"count":0}
	key = ""
	if 'user-plans' in session :
		plans = session['user-plans'] ;
		amount = [plan['amount'] for plan in plans if plan['amount'] > 0]
		if len(amount) > 0:
			key = CONFIG['stripe']['pub'].strip()
			bill['count'] = len(amount)
			bill['amount']= sum(amount)
	return render_template('checkout.html',bill=bill,key=key)
"""
	This function is intended to performa an actual payment
"""
@app.route('/pay',methods=['POST'])
def pay():
	
	token 		= request.form['stripeToken']
	uid		= request.form['stripeEmail']
	tokenType	= request.form['stripeTokenType']

	DB = COUCHDB.get_db(CONFIG['couchdb']['db']) ;
	handler = Domain.User(DB,stripe) ;
	#
	# Let's insure the preconditions are met i.e
	#	card,invoice
	info	= session['user-info']
	uid	= info['_id']
	if 'sources' not in info or token not in info['sources']:
		#
		# Let's create the card
		handler.save_card(uid,token)
	#
	# Let's create a charge here ...
	plans = session['user-plans']
	
	amount=[item['price'] for item in plans if item['status'] == 'past_due' ]
	if len(amount) == 0:
		amount = 0
	else:
		amount = sum(amount)
	handler.charge(uid,amount)
	session['user-info'] = handler.user
	return ('',204)
	#return render_template("bill.html",plans=plans, total_amount=amount)


@app.route('/bill',methods=['GET'])
def bill():
	return render_template('bill.html')
@app.route('/buy',methods=['POST'])
def buy():
	id = request.form['id'] ;
	
	if id in ['subscribe','now']:
		email = request.form['stripeEmail'] 
		token = request.form['stripeToken']
		user = Domain.User(DB,stripe,token) ;
		user.init(email) ;
		if user.exists() :
			print "Update with anything if need be"
			pass
		else:
			#
			# Create the user with the associated plan/or payment method
			#
			user.save() ;
			user.publish()
	else:
		pass
	return "0"
"""
	This function provides general information about service plans
	@TODO: provide filtering criteria
"""
@app.route('/plans',methods=['GET'])
def plans():
	plans = stripe.Plan.list().data	
	plans = [{"id":item.id,"price":item.amount/100,"feature":item.metadata['info'],"trial":item.trial_period_days} for item in plans if re.match('cloudplayer',item.id)]
	
	if 'uid' in request.headers and request.headers['uid'] != '':
		uid = request.headers['uid']
		DB = COUCHDB.get_db(CONFIG['couchdb']['db']) ;
		handler = Domain.User(DB)
		handler.init(uid) 
		myplans = [{"id":item.plan.id,"price":item.plan.amount/100,"feature":item.plan.metadata['info'],"status":item.status} for item in handler.subscriptions()]
		keys = [plan['id'] for plan in myplans]
		plans = [plan for plan in plans if plan['price'] > 0 and plan['id'] not in keys]
		plans = {'myplans':myplans,"plans":plans}
		session['user-plans'] = myplans
	plans = json.dumps(plans)
	return plans		
"""
	This function subscribes a user to a given plan(s)
	If the plan is new, then we do NOT need a credit card info
	
"""
@app.route('/subscribe',methods=['POST'])
def subscribe():
	
	
	
	plans = [{"id":id} for id in  request.get_json(silent=True)]

	user = session['user-info']
	
	DB = COUCHDB.get_db(CONFIG['couchdb']['db']) ;
	handler = Domain.User(DB,stripe)
	r = handler.subscribe(user['id'],plans)
	return json.dumps(r)


app.debug = True ;
app.secret_key = '360-8y-[0v@t10n]+kr81v17y'
app.run(port=PORT,threaded=True)
